/*	BlockBlast.c
	Simple block removing game
	(C)2011 Morgan Evans */
	
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_mixer.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "theme.h"
#include "block.h"
#include "utils.h"
#include "explosion.h"

/*  Change this to set the delay (in seconds) before the rows automatically move
	up. */
const int MOVE_TIME = 2;
const int FPS = 30;

SDL_Surface * initSDL() {
	// Initialise SDL library, if we can't terminate the program.
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0) {
		fprintf(stderr,"Failed to initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}
	
	// Set the window title
	SDL_WM_SetCaption("BlockBlast - alpha build 16072012", NULL);
	
	// Init the SDL font library
	if (TTF_Init() == -1) {
		printf("Failed to initialize TTF library. %s\n", TTF_GetError());
		exit(1);
	}
	
	// Init SDL_image library
	int img_init_flags = IMG_INIT_JPG | IMG_INIT_PNG;
	int img_init = IMG_Init(img_init_flags);
	if (img_init&img_init_flags != img_init_flags) {
		printf("Failed to initialize SDL_image library. %s\n", IMG_GetError());
		exit(1);
	}
	
	// Open audio mixer at required rate, etc.
	if (Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 1024) == -1) {
		printf("Failed to open audo mixer. %s\n", Mix_GetError());
		exit(1);
	}
	
	// Make sure SDL exits cleanly when the program terminates.
	atexit(SDL_Quit);
	
	SDL_Surface * screen = SDL_SetVideoMode(640, 480, 32, SDL_ANYFORMAT);
	if (screen == NULL) {
		fprintf(stderr,"Failed to set up display: %s\n", SDL_GetError());
		exit(2);
	}
	
	return screen;
}

void cleanQuit() {
	// Release resources used by SDL_image
	IMG_Quit();
	
	// Release resources used by SDL_mixer
	Mix_CloseAudio();
	
	// Exit and free remaining SDL resources
	exit(0);
}

char * getScoreString(int value, int max_digits) {
	char * text_value = (char *) malloc(sizeof(char) * (max_digits + 1));
	int i,j;
	for (i = 0; i <= max_digits; i++) text_value[i] = 0;
	sprintf(text_value, "%d", value);
	int str_len = strlen(text_value);
	i = max_digits - 1;
	for (j = str_len - 1; j >= 0; j--) {
		text_value[i--] = text_value[j];
	}
	for (j = 0; j <= i; j++) text_value[j] = '0';
	text_value[max_digits] = 0;
	return text_value;
}	// Returns the player score in a specially formatted (zero prefixed) string.

int main(int argc, char *argv[]) {
	int i,j,n; // These variables are usual used as indexes in loops.
	//	The variable below defines how long to wait before scrolling blocks up.
	int move_timer = MOVE_TIME * FPS;
	/*	randx and randy are used to create a pixel offset for "shaking" the
		blocks in the stack. */
	int randx = 0, randy = 0;
	/*	shake_timer is actually a counter which triggers the shake animation 
		after the stack has reached maximum height and to terminate the game if
		the user doesn't reduce the height of the pile within a certain time. */
	int shake_timer = 0;
	/*	marked is used to count the number of blocks that the define the pattern
		of colour the user has clicked on. */
	int marked;
	/*	click_timer is used to set a delay between clicks to prevent accidental
		and worthless double (or more) click mouse events. */
	int click_timer = 0;
	/*	mousex and mousey are used to get the position of the mouse when the 
		user clicks the select (left-hand) button */
	int mousex, mousey;
	/*	score, keep track of your current score, naturally. old_score is used 
	 *	to update the SDL_surface showing the score when it changes. */
	int score = 0, old_score = -1;
	
	// Set up a 640 x 480 display, if we can't terminate the program.
	SDL_Surface * screen = initSDL();
	Game_Theme * theme = loadTheme("default/", screen);
	
	/* 	Create storage area for a full set of block structure pointers (15 rows 
		of 12 columns, last row hidden) */
	Block * blocks = (Block *) malloc(sizeof(Block) * 12 * 15);
	int colour_max = initBlocks(blocks, theme);
	
	// Randomize the next block colour
	int block_colour = rand() % colour_max;
	
	// Create surfaces to render score onto.
	SDL_Surface * score_surface_fg = NULL, * score_surface_bg = NULL;

	// Reserve memory for a lot of explosions.
	Explosion ** explosions = (Explosion **) malloc(sizeof(Explosion *) * 240);
	for (i = 0; i < 240; i++) {
		explosions[i] = NULL;
	}
	
	// Start playing game music
	if (Mix_FadeInMusic(theme->game_music, -1, 2000) == -1) {
		printf("Failed to play game music. %s\n", Mix_GetError());
		exit(3);
	}
	
	/*	rect is used for two purposes, the first is to define a clipping area
		for the play area, to prevent blocks from covering areas of the display
		they shouldn't, its second purpose is to define where to draw each 
		block. */
	SDL_Rect rect;
	rect.w = rect.h = 32;
	
	while (shake_timer < 2 * FPS) {
		// Get user input
		SDL_PumpEvents();
		if ((SDL_GetMouseState(&mousex, &mousey) & SDL_BUTTON(1))
			&& click_timer == 10) {
			// The user's clicked, let's see if it's over a block.
			for (i = 0; i < 12 * 14; i++) {
				if (mousex >= blocks[i].x && mousex < blocks[i].x + 32 &&
					mousey >= blocks[i].y && mousey <= blocks[i].y + 32) {
					/*	It is, so mark every occurance of a block that's 
						connected by colour and grouping.*/ 
					marked = 0;
					matchBlock(i, blocks, -1, &marked);
					if (marked >= 3) {
						int count = 0;
						/* If more than 3 blocks in group, remove them, increase 
							the players score and initialise explosions for the 
							blocks. */
						for (j = 0; j < 12 * 15; j++) {
							if (blocks[j].marked) {
								// Make block invisible and unmark it
								blocks[j].visible = 0;
								blocks[j].marked = 0;
								// Increment score count
								count++;
								// Increment score.
								score += count * 10;
								// Register explosion if space is available.
								for (n = 0; n < 240; n++) {
									if (explosions[n] == NULL) {
										explosions[j] =
											createExplosion(blocks[j].x,
											blocks[j].y, blocks[j].image);
										break;
									}
								}
							}
						}
						Mix_PlayChannel(1, theme->match_sound, 0);
					} else {
						// ... otherwise unmark them and leave them alone.
						for (j = 0; j < 12 * 15; j++) {
							blocks[j].marked = 0;
						}
						Mix_PlayChannel(1, theme->click_sound, 0);
					}
					// Exit the loop as we've clicked over a block.
					break;
				}
			}
					
			// Reset the click timer to wait for the next click event.
			click_timer = 0;
		}
		
		// Get keyboard input currently check for space which auto-scrolls.
		unsigned char * key_state = SDL_GetKeyState(NULL);
		if (key_state[SDLK_SPACE] && move_timer >= 0) {
			move_timer = -1;
		}
		
		// Remove any surface clipping...
		SDL_SetClipRect(screen, NULL);
		// ... and draw the background in.
		SDL_BlitSurface(theme->background, NULL, screen, NULL);
		
		// Update the score rendering surfaces if the score has changed.
		if (score != old_score) {
			if (score_surface_fg != NULL) {
				SDL_FreeSurface(score_surface_fg);
				SDL_FreeSurface(score_surface_bg);
			}
			char * score_text = getScoreString(score, 8);
			score_surface_fg = TTF_RenderText_Solid(theme->value_font, score_text,
				theme->value_colours[0]);
			score_surface_bg = TTF_RenderText_Solid(theme->value_font, score_text,
				theme->value_colours[1]);
			free(score_text);
			old_score = score;
		}
		
		// Draw the score on screen.
		rect.x = 414 + ((224 - score_surface_fg->w) / 2);
		rect.y = 18;
		rect.w = score_surface_fg->w;
		rect.h = score_surface_fg->h;
		SDL_BlitSurface(score_surface_bg, NULL, screen, &rect);
		rect.x -= 2;
		rect.y -= 2;
		SDL_BlitSurface(score_surface_fg, NULL, screen, &rect);
		
		// Draw a semi-transparent filled rectangle that goes under the play area.
		// boxRGBA(screen, 12, 12, 404, 468, 0, 0, 0, 96);
		filledRectAlpha(screen, 12, 12, 404, 468, 0x000000A0);
		// Clip to the play area where we're going to draw the blocks.
		rect.x = 16;
		rect.y = 16;
		rect.w = 2 + (32 * 12);
		rect.h = 2 + (32 * 14);
		SDL_SetClipRect(screen, &rect);
		
		/*	Here we're handling the automatic movement of the columns of blocks
			up the screen.  The delay for a new row to appear is defined by the
			variable 'MOVE_TIME' set in the variables at the top of the main
			function. */
		int max_height = moveBlocksUp(blocks, &block_colour, &move_timer);
		// If not, keep the shake warning timer zeroed.
		if (!max_height) {
			shake_timer = 0;
			Mix_HaltChannel(2);
		} else {
			if (shake_timer == 0) {
				Mix_PlayChannel(2, theme->alarm_sound, -1);
			}
			// Otherwise increment the shake warning timer.
			shake_timer++;
			if (move_timer < MOVE_TIME * FPS) {
				move_timer = MOVE_TIME * FPS;
			}
		}
		
		// Draw the blocks in.
		rect.w = 32;
		rect.h = 32;
		for (i = 0; i < 12 * 14; i++) {
			if (blocks[i].visible) {
				if (shake_timer > 0) {
					/* 	If the shake timer is incrementing, we need to 
					 * 	shake the blocks to warn the user that the pile 
					 * 	is at a critial point. */
					randx = -1 + (rand() % 3);
					randy = -1 + (rand() % 3);
				}
				rect.x = blocks[i].x + randx;
				rect.y = blocks[i].y + randy;
				if (rect.y < 464) {
					SDL_BlitSurface(theme->block_images[blocks[i].image], NULL, 
					screen, &rect);
				}
			}
		}
		
		// Animate any explosions and draw them in.
		rect.w = 16;
		rect.h = 16;
		for (n = 0; n < 240; n++) {
			if (explosions[n] != NULL) {
					for (i = 0; i < 4; i++) {
						rect.x = explosions[n]->peices[i].x;
						rect.y = explosions[n]->peices[i].y;
						SDL_BlitSurface(
							theme->exploding_images[explosions[n]->block_colour][i],
							NULL, screen, &rect);
					}
					animateExplosion(explosions[n]);
					if (explosionDone(explosions[n])) {
						free(explosions[n]);
						explosions[n] = NULL;
					}
			}
		}
		
		// Move the blocks down if there's nothing below them
		dropBlocksDown(blocks);
		
		// Update the surface to display the background and blocks.	
		SDL_UpdateRect(screen, 0, 0, 0, 0); 
		
		// Maintain a rough estimate of the desired frame rate.
		SDL_Delay(1000 / FPS);
		
		// Decrement the automatic move timer.
		move_timer--;
		if (move_timer == -9) move_timer = MOVE_TIME * FPS;
		
		// Increment the click timer.
		if (click_timer < 10) click_timer++;
	}

	// Free surface memory used by score renderers
	SDL_FreeSurface(score_surface_bg);
	SDL_FreeSurface(score_surface_fg);
	
	// Free memory used by block structures.
	free(blocks);
	
	// Free memory used by explosions array
	free(explosions);
	
	// Free resources used by the theme
	freeTheme(theme);
	
	// Exit and free all remaining SDL resources.
	cleanQuit();
}
