/*	theme.c
	Theme handling for BlockBlast
	(C)2012 Morgan Evans */

#include <stdio.h>
#include <stdlib.h>	
#include "theme.h"
#include "utils.h"
#include "glob.h"

void __setMaskRGBValue(SDL_Surface * surface) {
	// Get the colour depth info from the surface.
	const int bpp = surface->format->BitsPerPixel;
	// Map the colour of the top left pixel as the mask.
	if (bpp == 24 || bpp == 32) {
		unsigned char red = *(unsigned char *) surface->pixels;
		unsigned char green = *(unsigned char *) (surface->pixels + 1);
		unsigned char blue = *(unsigned char *) (surface->pixels + 2);
		SDL_SetColorKey(surface, SDL_SRCCOLORKEY | SDL_RLEACCEL,
			SDL_MapRGB(surface->format, red, green, blue));
	}
}

SDL_Surface * __createTiledBackground(SDL_Surface * tile_surface, 
	const int width, const int height, const SDL_Surface * screen) {
	SDL_Surface * background_image = SDL_CreateRGBSurface(screen->flags, 640, 
			480, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, 
			screen->format->Bmask, screen->format->Amask);
	int x, y;
	SDL_Rect rect;
	rect.w = tile_surface->w;
	rect.h = tile_surface->h;
	for (x = 0; x < width; x += tile_surface->w) {
		for (y = 0; y < height; y += tile_surface->h) {
			rect.x = x;
			rect.y = y;
			SDL_BlitSurface(tile_surface, NULL, background_image,
				&rect);
		}
	}
	
	return background_image;
}

SDL_Surface * _loadGameBackground(const char * theme_folder, const SDL_Surface * screen) {
	// Create the complete relative path to the image file.
	char * background_image_file = mergeStrings(theme_folder, "background.jpg");
	
	// Load the image into an SDL_Surface
	SDL_Surface * temp_background_image = IMG_Load(background_image_file);
	if (!temp_background_image) {
		if (strcasecmp(theme_folder, "./themes/default/") == 0) {
			printf("Failed to load any background image: %s\n", IMG_GetError());
			exit(2);
		}
		temp_background_image = _loadGameBackground("./themes/default/", screen);
	}
	// background_image will be returned to the calling function
	SDL_Surface * background_image = NULL;
	// Check the size of the loaded image.
	if (temp_background_image->w < 640 || temp_background_image->h < 480) {
		/*	It's smaller than the window size, so we create an appropriately 
			sized background and tile the loaded image all over it. */
		background_image = __createTiledBackground(temp_background_image, 640,
			480, screen);
		SDL_FreeSurface(temp_background_image);
	} else background_image = temp_background_image;

	// Free memory allocated for background_image_file string.
	free(background_image_file);
	
	return background_image;
}	// Loads the game background image from the chosen theme folder.

TTF_Font * _loadFont(const char * theme_folder, const char * filename, const int size) {
	const char * full_path = mergeStrings(theme_folder, filename);
	TTF_Font * font = TTF_OpenFont(full_path, size);
	if (!font) {
		if (strcasecmp(theme_folder, "./themes/default/") == 0) {
			printf("Failed to load any font.\n");
			exit(2);
		}
		font = _loadFont("./themes/default/", filename, size);
	}
	
	return font;
}

Mix_Music * _loadMusic(const char * theme_folder, const char * filename) {
	const char * full_path = mergeStrings(theme_folder, filename);
	Mix_Music * music = Mix_LoadMUS(full_path);
	if (!music) {
		if (strcasecmp(theme_folder, "./themes/default/") == 0) {
			printf("Failed to load music file %s.\n", filename);
			exit(2);
		}
		music = _loadMusic("./themes/default/", filename);
	}
	
	return music;
}

Mix_Chunk * _loadSample(const char * theme_folder, const char * filename) {
	const char * full_path = mergeStrings(theme_folder, filename);
	Mix_Chunk * sample = Mix_LoadWAV(full_path);
	if (!sample) {
		if (strcasecmp(theme_folder, "./themes/default/") == 0) {
			printf("Failed to load sample file %s.\n", filename);
			exit(2);
		}
		sample = _loadSample("./themes/default/", filename);
	}
	
	return sample;
}

void _loadBlocks(const char * theme_folder, SDL_Surface * screen, Game_Theme * theme) {
	// Get the list of files than match the pattern *_block.png
	const char * full_pattern = mergeStrings(theme_folder, "*_block.png");
	glob_t g;
	int glob_res = glob(full_pattern, 0, 0, &g);
	
	// Set the block image count in the Game_Theme structure (theme)
	theme->block_image_count = g.gl_pathc;
	
	/* 	Reserve enough memory to hold the appropriately size array of 
		SDL_Surface pointers for all the blocks in the theme folder. */
	theme->block_images = (SDL_Surface **) malloc(sizeof(SDL_Surface *) *
		theme->block_image_count);
		
	// Load the images into the block_images SDL_Surface pointer array.
	int i;
	for (i = 0; i < theme->block_image_count; i++) {
		theme->block_images[i] = IMG_Load(g.gl_pathv[i]);
		if (!theme->block_images[i]) {
			printf("Failed to load block image. %s\n", IMG_GetError());
			exit(2);
		} else {
			// Use the top left pixel colour as the mask.
			__setMaskRGBValue(theme->block_images[i]);
		}
	}
	
	/* 	Reserve memory to hold the four block segments used in the explosion 
		animation. */
	theme->exploding_images = (SDL_Surface ***) malloc(sizeof(SDL_Surface **) *
		theme->block_image_count);
	for (i = 0; i < theme->block_image_count; i++) {
		theme->exploding_images[i] =
			(SDL_Surface **) malloc(sizeof(SDL_Surface *) * 4);
	}
	
	// Divide the block images into four segment surfaces to use in explosion.
	int j;
	for (i = 0; i < theme->block_image_count; i++) {
		int segment_width = theme->block_images[i]->w / 2;
		int segment_height = theme->block_images[i]->h / 2;
		for (j = 0; j < 4; j++) {
			theme->exploding_images[i][j] =
				SDL_CreateRGBSurface(SDL_SWSURFACE, segment_width, segment_height,
					theme->block_images[i]->format->BitsPerPixel,
					theme->block_images[i]->format->Rmask,
					theme->block_images[i]->format->Gmask,
					theme->block_images[i]->format->Bmask,
					theme->block_images[i]->format->Amask);
			SDL_SetColorKey(theme->exploding_images[i][j],
				SDL_SRCCOLORKEY | SDL_RLEACCEL,
				theme->block_images[i]->format->colorkey);
		}
		
		SDL_Rect r;
		r.x = 0; r.y = 0; r.w = segment_width; r.h = segment_height;
		SDL_BlitSurface(theme->block_images[i], &r, theme->exploding_images[i][0],
			NULL);
		r.x = segment_width;
		SDL_BlitSurface(theme->block_images[i], &r, theme->exploding_images[i][1],
			NULL);
		r.x = 0; r.y = segment_height;
		SDL_BlitSurface(theme->block_images[i], &r, theme->exploding_images[i][2],
			NULL);
		r.x = segment_width;
		SDL_BlitSurface(theme->block_images[i], &r, theme->exploding_images[i][3],
			NULL);
	}
	
	// Free directory list resources
	globfree(&g);
}

Game_Theme * loadTheme(const char * theme_name, SDL_Surface * screen) {
	const char * theme_path = mergeStrings("./themes/",theme_name);
	Game_Theme * theme = (Game_Theme *) malloc(sizeof(Game_Theme));
	// Load background surface
	theme->background = _loadGameBackground(theme_path, screen);
	// Load block image surfaces
	_loadBlocks(theme_path, screen, theme);
	// Load font used for values
	theme->value_font = _loadFont(theme_path, "values.ttf", 48);
	// Reserve memory for value font colours.
	theme->value_colours = (SDL_Color *) malloc(sizeof(SDL_Color) * 2);
	// Set value font colours (todo: load from file)
	int i;
	unsigned char * c = (unsigned char *) theme->value_colours;
	for (i = 0; i < 3; i++) {
		c[i] = 255;
		c[i+4] = 0;
	}
	// Load font used for labels
	// theme->label_font = _loadFont(theme_path, "values.ttf", 20);  //todo
	// Reserve memory for label font colours.
	theme->label_colours = (SDL_Color *) malloc(sizeof(SDL_Color) * 2);
	// Set label font colours (todo: load from file)
	c = (unsigned char *) theme->label_colours;
	for (i = 0; i < 3; i++) {
		c[i] = 255;
		c[i+4] = 0;
	}
	// Load game music
	theme->game_music = _loadMusic(theme_path, "music.ogg");
	// Load click sound sample
	theme->click_sound = _loadSample(theme_path, "click.wav");
	theme->match_sound = _loadSample(theme_path, "match.wav");
	theme->alarm_sound = _loadSample(theme_path, "alarm.wav");
	return theme;
}

void freeTheme(Game_Theme * theme) {
	// Free background surface
	SDL_FreeSurface(theme->background);
	
	// Free block image surfaces
	int i;
	for (i = 0; i < theme->block_image_count; i++) {
		SDL_FreeSurface(theme->block_images[i]);
	}
	
	// Free memory allocated to block image surface array
	free(theme->block_images);
	
	// Free explosion animation image surfaces
	int j;
	for (i = 0; i < theme->block_image_count; i++) {
		for (j = 0; j < 4; j++) {
			SDL_FreeSurface(theme->exploding_images[i][j]);
		}
		// Free memory allocated to surfaces pointers for each segment peice.
		free(theme->exploding_images[i]);
	}
	
	// Free memory allocated to explosion image array
	free(theme->exploding_images);
	
	// Free fonts
	TTF_CloseFont(theme->value_font);
	// TTF_CloseFont(theme->label_font);
	
	// Free memory reserved for font colours.
	free(theme->value_colours);
	free(theme->label_colours);
	
	// Free music
	Mix_FreeMusic(theme->game_music);
	
	// Free sound samples
	Mix_FreeChunk(theme->click_sound);
	Mix_FreeChunk(theme->match_sound);
	Mix_FreeChunk(theme->alarm_sound);
}
