/*	title.c
	Title screen and game configuration interface
	(C)2012 Morgan Evans */
	
#include "block.h"
#include "utils.h"
#include "ui.h"
#include "explosion.h"
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <time.h>

#define STAGE_SHOW_TITLE 0
#define STAGE_SHAKE_TITLE 1
#define STAGE_BLOW_TITLE 2

char ** createThemesList(int * list_size) {
	// Open the themes directory for reading.
	DIR * d = opendir("./themes");
	if (!d) {
		printf("Failed to open themes folder.");
		exit(1);
	}
	
	/*	Rather than messing about with linked lists or reallocating, there will 
		be a maximum limit of 256 themes allowed in the themes directory.  */
	char ** list = (char **) malloc(sizeof(char *) * 256);
	*list_size = 0;
	
	struct dirent * de;
	while (de = readdir(d)) {
		// Make sure file is a directory and doesn't start with a period (.)
		if (de->d_type == DT_DIR && de->d_name[0] != '.') {
			// Get length of the file name.
			const int fname_len = strlen(de->d_name);
			// Reserve enough memory to copy the name into the list.
			list[*list_size] = (char *) malloc(sizeof(char) * (fname_len + 2));
			// Copy the name into the list.
			strcpy(list[*list_size], de->d_name);
			// Append '/' and terminator to end of string.
			list[*list_size][fname_len] = '/';
			list[*list_size][fname_len+1] = 0;
			// Increment the list size by one.
			*list_size = *list_size + 1;;
		}
	}
	
	closedir(d);
	
	return list;
}

void freeThemesList(char ** list, const int list_size) {
	int i;
	
	// Free memory reserved for each name in the list.
	for (i = 0; i < list_size; i++) {
		free(list[i]);
	}
	
	// Free memory allocated for name pointers.
	free(list);
}

void showRandomBackgroundBlock(Block * blocks, const int VISIBLE_COUNT) {
	// Seed random number generator
	srand(time(NULL));
	
	while (VISIBLE_COUNT < 300) {
		// Pick a random block
		int offset = rand() % 300;
		// Check to see if it's visible
		if (!blocks[offset].visible) {
			// It's not, so give it a random chance to become visible
			int chance = rand() % 10;
			if (chance == 1) {
				// It's got it's chance, make it visible
				blocks[offset].visible = 1;
				break;
			}
		}
	}
}

int main(int argc, char ** argv) {
	int list_size, active_theme_index = 0, quit = 0;
	char ** list = createThemesList(&list_size);

	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);
	
	// Set the window title
	SDL_WM_SetCaption("BlockBlast - Title Test", NULL);
	
	// Init the SDL font library
	if (TTF_Init() == -1) {
		printf("Failed to initialize TTF library. %s\n", TTF_GetError());
		exit(1);
	}
	
	// Init SDL_image library
	int img_init_flags = IMG_INIT_JPG | IMG_INIT_PNG;
	int img_init = IMG_Init(img_init_flags);
	if (img_init&img_init_flags != img_init_flags) {
		printf("Failed to initialize SDL_image library. %s\n", IMG_GetError());
		exit(1);
	}
	
	// Open audio mixer at required rate, etc.
	if (Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 1024) == -1) {
		printf("Failed to open audo mixer. %s\n", Mix_GetError());
		exit(1);
	}
	
	SDL_Surface * screen = SDL_SetVideoMode(640, 480, 32, SDL_ANYFORMAT);
	Game_Theme * theme = loadTheme(list[active_theme_index], screen);
	
	/* 	Choose three colours, one for "Block" text, the other for "Blast" text 
		and the other for the background tiles. */
	srand(time(NULL));
	int col = rand() % theme->block_image_count;
	int cols[] = {	col,
					(col + 1) % theme->block_image_count,
					(col + 2) % theme->block_image_count };
					
	/* 	We're going to create a 20 x 15 grid of blocks and set their positions 
		so they cover the whole screen and define their colour using the array 
		below. */
	unsigned char pattern[] = {	0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								1,1,0,0, 1,0,0,0, 0,1,0,0, 0,1,0,0, 1,0,1,0,
								1,0,1,0, 1,0,0,0, 1,0,1,0, 1,0,1,0, 1,0,1,0,
								1,1,0,0, 1,0,0,0, 1,0,1,0, 1,0,0,0, 1,1,0,0,
								1,0,1,0, 1,0,0,0, 1,0,1,0, 1,0,1,0, 1,0,1,0,
								1,1,0,0, 1,1,1,0, 0,1,0,0, 0,1,0,0, 1,0,1,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,2,2,0, 0,2,0,0, 0,0,2,0, 0,0,2,2, 0,2,2,2,
								0,2,0,2, 0,2,0,0, 0,2,0,2, 0,2,0,0, 0,0,2,0,
								0,2,2,0, 0,2,0,0, 0,2,2,2, 0,0,2,0, 0,0,2,0,
								0,2,0,2, 0,2,0,0, 0,2,0,2, 0,0,0,2, 0,0,2,0,
								0,2,2,0, 0,2,2,2, 0,2,0,2, 0,2,2,0, 0,0,2,0 };
			
	Block * blocks = (Block *) malloc(sizeof(Block) * 20 * 15);
	int x, y, j, i = 0;
	for (y = 0; y < 480; y += 32) {
		for (x = 0; x < 640; x += 32) {
			blocks[i].x = x;
			blocks[i].y = y;
			blocks[i].image = (i < 240) ? cols[pattern[i]] : cols[0]; 
			blocks[i].visible = (i < 240) ? (pattern[i] > 0) ? 1 : 0 : 0;
			blocks[i].marked = 0;
			i++;
		}
	}
	
	// shake_timer is used to control when blocks start to shake.
	int shake_timer = 0;
	
	// Create a menu strip
	TTF_Font * font = TTF_OpenFont("./themes/default/values.ttf",32);
	Menu * menu = createMenu(96, 48, 48, 608, 48, font);
	addMenuItem(menu, "Single Player", NULL, NULL);
	addMenuItem(menu, "Multi Player", NULL, NULL);
	addMenuItem(menu, "High Scores", NULL, NULL);
	addMenuItem(menu, "Options", NULL, NULL);
	SDL_Surface * menu_surface;
	
	// scroll_dir determines if the menu items should scroll and in which direction.
	int scroll_dir = 0; // No scrolling if 0.
	
	// TEST CODE:  Sample explosions
	Explosion ** explosions = (Explosion **) malloc(sizeof(Explosion *) * 300);
	for (i = 0; i < 300; i++) {
		explosions[i] = NULL;
	}
	
	int animation_stage = STAGE_SHOW_TITLE;
	i = 0;
	SDL_Rect r;
	r.w = 32;
	r.h = 32;
	do {
		// Capture keypress events for the menu.
		SDL_PumpEvents();
		unsigned char * key_state = SDL_GetKeyState(NULL);
		if (key_state[SDLK_LEFT]) {
			scroll_dir = -1;
		}
		if (key_state[SDLK_RIGHT]) {
			scroll_dir = 1;
		}
		if (key_state[SDLK_SPACE]) break;
		
		// Clear the screen with black.
		boxColor(screen, 0, 0, 640, 480, 0x000000FF);			
		
		// Draw the blocks onto the title screen if they're visible
		int visible_count = 0;
		for (x = 0; x < 300; x++) {				
			if (blocks[x].visible) {
				r.x = blocks[x].x;
				r.y = blocks[x].y;
				// If we're at stage 2, alter their position a little to make them "shake"
				if (animation_stage > STAGE_SHOW_TITLE) {
					r.x += -1 + (rand() % 3);
					r.y += -1 + (rand() % 3);
				}
				SDL_BlitSurface(theme->block_images[blocks[x].image], NULL, 
					screen, &r);
				visible_count++;
			}
		}
		
		// If we're at stage 1, show random blocks until they're all visible.
		if (visible_count < 300 && animation_stage == STAGE_SHOW_TITLE) {
			showRandomBackgroundBlock(blocks, visible_count);
		}
		
		// If all 300 blocks are visible, we can go onto stage 2.
		if (visible_count == 300 && animation_stage == STAGE_SHOW_TITLE) {
			animation_stage = STAGE_SHAKE_TITLE;
			shake_timer = 0;
		}
		
		// If we're at stage two, increment the stage timer.
		if (animation_stage == STAGE_SHAKE_TITLE) shake_timer++;
		
		// If the shake timer reaches 60, we can start stage 3.
		if (shake_timer == 60 && animation_stage == STAGE_SHAKE_TITLE) {
			animation_stage = STAGE_BLOW_TITLE;
		}
		
		/* 	At stage 3 we start destroying random blocks until there are none 
		 	left. */
		if (animation_stage == STAGE_BLOW_TITLE && visible_count > 0) {
			// Find a visible block (one that's not been destroyed)
			int block_to_blow;
			do {
				block_to_blow = rand() % 300;
			} while (!blocks[block_to_blow].visible);
			
			// Create four new explosions and add them into the explosions array.
			for (i = 0; i < 4; i++) {
				for (j = 0; j < 300; j++) {
					if (explosions[j] == NULL) {
						explosions[j] = createExplosion(blocks[block_to_blow].x,
							blocks[block_to_blow].y, blocks[block_to_blow].image);
						blocks[block_to_blow].visible = 0;
						break;
					}
				}
			}
		}
		
		// TEST CODE:  Draw explosions
		if (animation_stage == STAGE_BLOW_TITLE) {
			r.w = 16;
			r.h = 16;
			for (j = 0; j < 300; j++) {
				if (explosions[j] != NULL) {
					for (i = 0; i < 4; i++) {
						r.x = explosions[j]->peices[i].x;
						r.y = explosions[j]->peices[i].y;
						SDL_BlitSurface(theme->exploding_images[explosions[j]->block_colour][i],
							NULL, screen, &r);
					}
					animateExplosion(explosions[j]);
					if (explosionDone(explosions[j])) {
						free(explosions[j]);
						explosions[j] = NULL;
					}
				}
			}
		}
		
		// Draw menu
		r.x = 320 - (menu->width / 2);
		r.y = 431 - (menu->height / 2);
		r.w = menu->width;
		r.h = menu->height;		
		menu_surface = getRenderedMenu(menu, screen);
		SDL_BlitSurface(menu_surface, NULL, screen, &r);			
		
		// Scroll the menu
		scroll_dir = scrollMenuItems(menu, scroll_dir);
		
		// Update the screen surface to show its new contents.
		SDL_UpdateRect(screen, 0, 0, 0, 0);
		
		/* Go to next row for adding random blocks.
		y++;
		if (y > 14) y = 0; */
		
		// Wait for 1/40th second/	
		SDL_Delay(25);
		
		// Free the SDL_Surface representing the menu stripas it's updated each cycle.
		SDL_FreeSurface(menu_surface);
	} while (1);

	// Free any active explosions
	for (j = 0; j < 300; j++) {
		if (explosions[j] != NULL) {
			freeExplosion(explosions[j]);
			explosions[j] = NULL;
		}
	}
	
	// Free explosions array.
	free(explosions);

	// Free active theme	
	freeTheme(theme);
	
	// Free themes list
	freeThemesList(list, list_size);
	
	// Free and font resources
	TTF_CloseFont(font);
	
	// Release resources used by SDL_image
	IMG_Quit();
	
	// Release resources used by SDL_mixer
	Mix_CloseAudio();
	
	atexit(SDL_Quit);
	return 0;
}
