/*	ui.c
	Block Blast UI functionality.
	(C)2012 Morgan Evans */

#include "ui.h"
#include <string.h>
#include <math.h>

Menu_Item * createMenuItem(char * label, SDL_Surface * icon, void (*action)(),
	Menu_Item * previous_menu_item) {
	// If no label or icon passed, exit cleanly with NULL pointer.
	if ((label == NULL || strlen(label) == 0) && icon == NULL) return NULL;
	// Create storage for new Menu_Item structure and get it's pointer.
	Menu_Item * i = (Menu_Item *) malloc(sizeof(Menu_Item));
	// Get the length of the label.
	int label_len = strlen(label);
	// Reserve memory to copy the label into the Menu_Item structure.
	i->label = (char *) malloc(sizeof(char) * (label_len + 1));
	// Copy the label text into the Menu_Item structure.
	strcpy(i->label, label);
	// Copy references to icon SDL_surface and the item's function.
	i->icon = icon;
	i->action = action;
	/*	If there is a preceeding Menu_Item, set it's next Menu_Item pointer to 
		this Menu_Item. */
	if (previous_menu_item != NULL) previous_menu_item->next_menu_item = i;
	// Set this item's next menu item pointer to NULL;
	i->next_menu_item= NULL;
	// Return pointer to this Menu_Item.
	return i;
}

Menu_Item * freeMenuItem(Menu_Item * i) {
	// Exit cleanly if NULL pointer passed;
	if (i == NULL) return NULL;
	/*	We don't free the SDL_Surface used for the icon, nor the function as
		they may be refered to elsewhere/later.  Instead just NULL their values. */
	i->icon = NULL;
	i->action = NULL;
	//	We can free memory used for the label though
	free(i->label);
	//  Free rendered label surface if created.
	if (i->rendered_label != NULL) SDL_FreeSurface(i->rendered_label);
	//  Set the rendering position to 0;
	i->x = 0;
	// Record reference to next menu item so we can pass it back.
	Menu_Item * m = i->next_menu_item;
	//	NULL the pointer to the next Menu_Item structure
	i->next_menu_item = NULL;
	//	Free any memory reserved for the Menu_Item structure itself.
	free(i);
	// Pass back reference to next menu item.
	return m;
}

Menu * createMenu(unsigned char red, unsigned char green, unsigned char blue,
	unsigned short width, unsigned short height, TTF_Font * font) {
	// Return NULL if nonsense width and height parameters passed.
	if (width < 32 || width > 640 || height < 32 || height > 480) return NULL;
	// Reserve memory for Menu structure
	Menu * m = (Menu *) malloc(sizeof(Menu));
	// Set background colour for menu.
	m->background_colour[0] = red;
	m->background_colour[1] = green;
	m->background_colour[2] = blue;
	// Set the background image to NULL
	m->background_image = NULL;
	// Set the width of the menu in pixels.
	m->width = width;
	// Set the height of the menu in pixels.
	m->height = height;
	// Set the font used for menu text
	m->font = font;
	// Indicate that there are no current menu items in the menu.
	m->first_menu_item = NULL;
	m->menu_item_count = 0;
	// Set the selected menu item to be the first menu item.
	m->selection = 0;
	// Set minimum and maximum scroll offsets to zero
	m->min_x = m->max_x = 0;
	// Set scroll direction to zero (no movement)
	m->scroll_dir = 0;
	// Set scroll steps to zero (menu items in place)
	m->scroll_steps = 0;
	// Return pointer to Menu
	return m;
}

void freeMenu(Menu * m) {
	// Exit cleanly if NULL pointer passed.
	if (m == NULL) return;
	// We can't really do much about the background colour so leave it.
	// NULL the background_image reference
	m->background_image = NULL;
	// Make the menu width and height zero pixels
	m->width = m->height = 0;
	// Remove reference to font
	m->font = NULL;
	// Release any resources used for menu items.
	if (m->menu_item_count > 0) {
		Menu_Item * mi = m->first_menu_item;
		while (mi = freeMenuItem(mi));
	}
	// Set menu item count to 0
	m->menu_item_count = 0;
	// Set the selected menu item to -1.
	m->selection = -1;
	// Zero scrolling paramters
	m->min_x = m->max_x = m->scroll_dir = m->scroll_steps = 0;
	// Free memory reserved for Menu structure
	free(m);
}

void addMenuItem(Menu * menu, char * label, SDL_Surface * icon, void (*action)()) {
	// Create new menu item from given parameters
	Menu_Item * m = createMenuItem(label, icon, action, NULL);
	if (menu->menu_item_count == 0) {
		// If first menu item, set it as such.
		menu->first_menu_item = m;
	} else {
		// If not first item, go to last item on menu...
		Menu_Item * em = menu->first_menu_item;
		while (em->next_menu_item != NULL) {
			em = em->next_menu_item;
		}
		// ... and set it's next menu item pointer to the new menu item.
		em->next_menu_item = m;
	}
	
	// Render the menu item label surface
	const SDL_Color c = {255, 255, 255};
	m->rendered_label = TTF_RenderText_Solid(menu->font, m->label, c);
	
	// Increment the menu item count.
	menu->menu_item_count++;
	
	// Position in menu items correctly for rendering in the menu.
	// Position first to mid-list from the centre out to the right.
	const int X_POS_INC = menu->width / 2;
	const int MIDDLE_INDEX = menu->menu_item_count / 2;
	int x = X_POS_INC;
	int i;
	for (i = 0; i <= MIDDLE_INDEX; i++) {
		Menu_Item * mi = getMenuItem(menu, i);
		mi->x = x;
		x += X_POS_INC;
	}
	
	// Set the maximum boundary for rotating rendered menu items.
	menu->max_x = x;
	
	// Any remaining items on the list are placed to the left of centre.
	x = 0;
	while (i < menu->menu_item_count) {
		Menu_Item * mi = getMenuItem(menu, i);
		mi->x = x;
		x -= X_POS_INC;
		i++;
	}
	
	// Set the minimum boundary for rotating rendered menu items.
	menu->min_x = x;
}

Menu_Item * getMenuItem(Menu * menu, int index) {
	// Return NULL if out of bounds index is passed.
	if (index >= menu->menu_item_count || index < 0) return NULL;
	Menu_Item * m = menu->first_menu_item;
	int i = 0;
	while (i < index && m != NULL) {
		m = m->next_menu_item;
		i++;
	}
	
	return m;
}

SDL_Surface * getRenderedMenu(Menu * menu, SDL_Surface * screen) {
	SDL_Surface * menu_surface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA,
	menu->width, menu->height, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
	// Define the menu background colour
	const unsigned int BOX_COLOUR = (menu->background_colour[0] << 16) |
		(menu->background_colour[1] << 8) | menu->background_colour[2];
	
	// Get pointer to first pixel
	unsigned int * pixel_ptr = (unsigned int *) menu_surface->pixels;
	
	// Fill menu in with the background colour.
	int offset = 0;
	while (offset < menu->width * menu->height) {
		pixel_ptr[offset++] = BOX_COLOUR;
	}
	
	// Render in the menu items
	Menu_Item * mi = menu->first_menu_item;
	SDL_Rect r;
	r.y = (menu->height - mi->rendered_label->h) / 2;
	while (mi != NULL) {
		r.x = mi->x - (mi->rendered_label->w / 2);
		r.w = mi->rendered_label->w;
		r.h = mi->rendered_label->h;
		SDL_BlitSurface(mi->rendered_label, NULL, menu_surface, &r);
		mi = mi->next_menu_item;
	}
	
	/* 	Set alpha gradient at the left/right edges. */
	offset = 0;
	int x, y, alpha;
	const int CURVE_WIDTH = menu->width / 3;
	const int CURVE_RIGHT = menu->width - CURVE_WIDTH;
	const double DEG2RAD = 180 / M_PI;
	for (y = 0; y < menu->height; y++) {
		for (x = 0; x < menu->width; x++) {
			if (x < CURVE_WIDTH) {
				double angle = (90 * (x / (double) CURVE_WIDTH)) / DEG2RAD;
				alpha = 255 * sin(angle);
			} else if (x >= CURVE_RIGHT) {
				double angle = (90 * ((x - CURVE_RIGHT) / (double) CURVE_WIDTH)) / DEG2RAD;
				alpha = 255 * cos(angle);
			} else alpha = 255;
			if (y % 3 == 0) {
				alpha -= 32;
				if (alpha < 48) alpha = 48;
			}
			pixel_ptr[offset] = (pixel_ptr[offset] & 0x00FFFFFF) | (alpha << 24);
			offset++;
		}
	}

	return menu_surface;
}

int scrollMenuItems(Menu * menu, int direction) {
	if (direction != menu->scroll_dir) {
		if (direction != 0 && menu->scroll_dir != 0) {
			menu->scroll_steps = 16 - menu->scroll_steps;
		}
		menu->scroll_dir = direction;
	}
	
	if (menu->scroll_dir != 0) {
		int j;
		for (j = 0; j < menu->menu_item_count; j++) {
			Menu_Item * mi = getMenuItem(menu, j);
			mi->x += (menu->width / 32) * menu->scroll_dir;
			if (direction < 0 && mi->x <= menu->min_x) {
				mi->x = menu->max_x - (menu->width / 2);
			} else if (direction > 0 && mi->x >= menu->max_x) {
				mi->x = menu->min_x + (menu->width / 2);
			}	
		}
		menu->scroll_steps++;
		
		if (menu->scroll_steps == 16) {
			menu->scroll_dir = 0;
			menu->scroll_steps = 0;
		}
	} 
	
	return menu->scroll_dir;
}


